# coding: utf-8

import json
import requests
import traceback
import datetime
import base64
import re
import sys
from pandas import DataFrame, Series
import pandas as pd
import numpy as np


class HsonBase():
    """
    根据开放平台获取数据，接口设计如下：
    1、接口权限：
        接口权限是通过token控制，目前是直接从另外个服务器获取，没有加密小心别漏出去
    """
    # host = 'sandbox.hscloud.cn'
    host = 'open.hscloud.cn'
    host_v1 = 'https://' + 'open.hscloud.cn' + '/quote/v1'
    host_v2 = 'https://' + 'open.hscloud.cn' + '/quote/v2'
    host_info_v2 = 'https://' + 'open.hscloud.cn' + '/info/v2'
    g_token = None

    # def getAccessToken_hson(self):
    #     '''
    #     废弃，使用下面的hack
    #     '''
    #     #     key='d952ac95-5875-4254-8cc6-e250b340d883'
    #     #     secret='8a59e2f4-3c32-46c3-a74b-8d031d6de2ab'
    #
    #     key = 'a9b86543-9dfd-4ceb-b37e-9e698bc6fb2f'
    #     secret = '41043417-fe94-438a-9700-9698b1c333e3'
    #     url = 'https://sandbox.hscloud.cn/oauth2/oauth2/token'
    #     # py3使用
    #     authoriz = 'Basic ' + base64.b64encode(bytes(key + ":" + secret, 'utf8')).decode()
    #     # py2使用
    #     #     authoriz = 'Basic '+ base64.b64encode(key+":"+secret).decode()
    #     print(authoriz)
    #     # 头部Authorization应填写对“app_key:app_secret”进行Base64编码后的字符串（不包括双引号，字符采用UTF-8编码）。
    #     my_headers = {'Host': self.host,
    #                   'Content-type': 'application/x-www-form-urlencoded',
    #                   'Authorization': authoriz,
    #                   'Accept-Encoding': 'gzip, deflate'
    #                   }
    #     # 参数“grant_type”应填写“client_credentials”
    #     body = {"grant_type": "client_credentials"}
    #
    #     try:
    #         r = requests.post(url, headers=my_headers, data=body, timeout=5)
    #         #         print(r.text)
    #         if (r.status_code == 200):
    #             rs = r.json()
    #             return rs["access_token"]
    #     except:
    #         print("get Authorization exception ", sys.exc_info())
    #     return 'error'

    def get_access_token_hack(self):
        """
            所有接口的权限获取，都依赖它
        :return:
        """
        if self.g_token is not None:
            return self.g_token
        url_token = 'http://www.ziruxing.com:9003/api/cjj/dtzq/data/token'
        my_headers = {}
        r = requests.get(url_token, headers=my_headers, params=None, timeout=5)
        if r.status_code == 200:
            rs = json.loads(r.text)
            token = rs['data']
            self.g_token = token
            return token

    def http_get_hsopen(self, url, params=None):
        """
        请求接口的基类，所有请求接口都调用这个get方法，不用担心权限的获取
        :param url:
        :param params:
        :return:
        """
        try:
            accessToken = self.get_access_token_hack()
            #                 https://open.hscloud.cn/quote/v1/real?en_prod_code=603031.SS,603268.SS,&fields=prod_name,last_px&access_token=87CCBDA1385E48989CB8D7C98AE1C47E201710141738555FBA0802&t=1507985332927
            my_headers = {
                'User-Agent': 'User-Agent:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.62 Safari/537.36',
                'host': self.host,
                'Authorization': 'Bearer ' + accessToken
            }
            #         params.update({'access_token':accessToken})
            r = requests.get(url, headers=my_headers, params=params, timeout=5)
            #         print(r.text)
            if (r.status_code == 200):
                rs = r.json()
                return rs
        except:
            print("get exception ", sys.exc_info())
        return 'error'

    def get_stock_id_hson(self, stock_code):
        """
        转换成股票代码id,600136.XSHG->6001361
        """
        code = stock_code
        idx = stock_code.find('.')
        if idx > 0:
            code = code[0:idx]
        if code.startswith("60"):
            code = code + ".SS";
        elif code.startswith("00"):
            code = code + ".SZ";
        elif code.startswith("30"):
            code = code + ".SZ";
        elif code.startswith("39"):  # 指数相关
            code = code + ".SZ";
        return code


class HsonData(HsonBase):
    """
    1、接口权限：
    2、获取交易日
        get_trade_days
    3、获取资金流数据
        3.1 天
        3.2 分钟
    4、获取K线数据
        4.1 天
        4.2 周
        4.3 月
        4.4 分钟(1,5,15)
        4.5 支持股票和指数
    5、获取所有股票列表
        5.1 获取所有列表
        5.2 获取大盘股 小盘股 创业板列表
        5.3
    6、获取板块列表
        6.1 概念板块列表
        6.2 行业板块列表
        6.3 指数列表
    7、根据板块获取成分股列表
        7.1 概念板块
        7.2 行业板块
        7.3 指数
    8、根据股票获取相关板块
        8.1 概念板块列表
        8.2 行业板块列表
        8.3 地域板块列表
    9、根据股票获取上市日期
    10、分红送转查询
        10.1 送股，转增，派息，除权日，股权登记日，分红日
    11、查询基本面
        11.1 市盈率，市净率，流通市值，总股本

    12、今日申购新股

    13、获取证券信息
        13.1 证券代码	、证券简称、上市日期、证券类型、证券市场、上市板块

    14、查询前10大流通股东

    15、当前行情报价
        15.1 open_px,high_px,low_px,last_px,business_amount,business_balance,offer_grp,bid_grp

    16、分时数据

    16、板块成份股排序（block/sort）

    17、关联代码（codes）



    恒生开放平台API文档：https://www.hscloud.cn/wiki/api/1_quote_v1_kline.html

    """

    def get_holiday(self, year, finance_mic='SS'):
        """
        获取节假日
        return:
            ['20160101', '20160102',...]
        """
        url = self.host_v1 + '/market/holiday'
        params = dict(finance_mic=finance_mic,
                      date=year)
        result = self.http_get_hsopen(url, params)
        return result['data']['en_holiday'].split(',')

    def get_trade_days(self, start_date, end_date):
        '''
        获取两个日期之间的交易日
        包含 start_date 格式2014-12-31
        不包含 end_date 格式2014-12-31
        :return
            [20170105,20170106,...]
        '''
        trade_days = []

        year1 = start_date.year
        year2 = end_date.year
        holidays = []
        for i in range(year2 - year1 + 1):
            holiday = self.get_holiday(year1 + i)
            holidays.extend(holiday)

        date1 = start_date
        while date1 < end_date:
            date1_str = datetime.datetime.strftime(date1, '%Y%m%d')
            if date1_str not in holidays:
                trade_days.append(date1_str)
            date1 = date1 + datetime.timedelta(days=1)
        return trade_days

    def get_stock_list(self):
        """
        获取证券列表
        :return:
        code,hq_type_code,name
        600000,ESA.M,浦发银行
        600004,ESA.M,白云机场
        """
        try:
            url = 'https://open.hscloud.cn/quote/v1/market/detail?finance_mic=SS'
            rs = self.http_get_hsopen(url)
            df_ss = pd.DataFrame(rs['data']['market_detail_prod_grp'])

            url = 'https://open.hscloud.cn/quote/v1/market/detail?finance_mic=SZ'
            rs = self.http_get_hsopen(url)
            df_sz = pd.DataFrame(rs['data']['market_detail_prod_grp'])
            df = pd.concat([df_ss, df_sz])
            df = df[df['hq_type_code'].str.contains('ESA')]
            # del df['hq_type_code']
            df.rename(columns={'prod_code': 'code', 'prod_name': 'name'}, inplace=True)
            df.set_index('code', inplace=True)
            return df
        except:
            return None

        def get_money_flow_day(self, stock_code):
            """
            获取资金流
            :param stock_code: '600570.SS'
            :return: change是万级，例如下面，小单流入5800万
                     type      change         in        out
                0  little   5895.7080  152308028   93350948
                1  medium   6192.0940  315173000  253252060
                2   large    975.7561  315513917  305756356
                3   super -13063.5581  155311846  285947427
            """

            url = self.host_v1 + '/fundflow'
            stock_code = self.get_stock_id_hson(stock_code)
            params = {"en_prod_code": stock_code, "get_type": "byorder"}
            rs = self.http_get_hsopen(url, params)
            df = pd.DataFrame(rs['data']['fundflow'][stock_code], columns=['in', 'out'])
            # money_type = ["little", "medium", "large", "super"]
            df['type'] = rs['data']['fundflow']['fields']
            df['change'] = (df['in'] - df['out']) / 10000
            df = df[['type', 'change', 'in', 'out']]
            return df

    def get_money_flow_minute(self, stock_code, date):
        """
        获取分钟级别的资金流

        :param stock_code: 600570.SS
        :param date:
        :param time:
        :return:
           super_in  super_out   large_in  large_out  medium_in  medium_out  \
min_time
930         2117598    6777589    1911000          0    2620975      613626
        """
        url = self.host_v2 + '/fundflow_trademin'
        stock_code = self.get_stock_id_hson(stock_code)
        params = {"prod_code": stock_code, "date": date}
        rs = self.http_get_hsopen(url, params)
        df = pd.DataFrame(rs['data']['fundflow_trademin_grp'], columns=rs['data']['fields'])
        # money_type = ["little", "medium", "large", "super"]
        df.set_index('min_time', inplace=True)
        return df

    def get_boards_of_stock(self, stock_code, type='GN'):
        """
        根据股票获取概念板块或者行业板块
        :param stock_code:
        :param type:
        :return:
        """
        stock_code = self.get_stock_id_hson(stock_code)
        url = self.host_v1 + '/block/query'
        params = {"prod_code": stock_code}
        data = self.http_get_hsopen(url, params)
        boards = data['data'][stock_code]
        filter_boards = []
        for board in boards:
            if board[2].find("." + type) != -1:
                filter_boards.append(board)
        return filter_boards

    def get_board_name_of_stock(self, stock):
        try:
            boards = self.get_boards_of_stock(stock)
            rs_str = ''
            for board in boards:
                rs_str += (board[1] + ',')
            return rs_str[0:-1]
        except:
            print("get_board_name_of_stock exception ", traceback.print_exc())
            return ''

    def get_kline(self,
                  stock_code,
                  date,
                  data_count=10,
                  candle_period=6,  # 默认日K
                  candle_mode=1,  # 默认前复权
                  get_type='offset',  # 默认按照便宜值查询
                  min_time=None,  # 分钟级别需要
                  fields=None  # 默认全部字段
                  ):
        '''
        获取K线数据，基类，需要封装成好用的
        其余参数参考：https://www.hscloud.cn/wiki/api/1_quote_v1_kline.html
        根据需要改写此函数
        stock_code：
            有且仅能有 1 个代码；证券代码包含交易所代码做后缀，作为该代码的唯一标识。如：600570.SS
        get_type:
            offset 按偏移查找；
            range 按日期区间查找；
            必须输入其中一个值
        candle_period:取值可以是数字1-9，表示含义如下：
            1：1分钟K线
            2：5分钟K线
            3：15分钟K线
            4：30分钟K线
            5：60分钟K线
            6：日K线
            7：周K线
            8：月K线
            9：年K线
        candle_mode:K线模式
            0：原始K线
            1：前复权K线
            2：后复权K线
        fields: 允许的字段：
            开盘价：open_px
            最高价：high_px
            最低价：low_px
            收盘价：close_px
            成交量：business_amount
            成交额：business_balance
            如果没有指定任何有效的字段，则返回所有字段
        date:不输入默认为当前日期；请求日K线时，如果输入日期，不返回该日期的K线
            get_type=offset时有效
        min_time:分钟 K 线的时间 HHMM,对于 短 周期 K 线 类型 使用(1min,5min 等)，
            不填写表示最新的市场时间，若填写必须同时填写 date 字段。
            请求分钟K线时，如果输入该字段，不返回输入分钟的K线
            仅在 get_type=offset 且candle_period=1~5（分钟 K线）时有效。
        data_count:需要取得的 K 线的根数
            如果该字段不存在，取值范围[1, 1000]，默认为 10 个。
            仅在 get_type=offset 时有效。
        start_date:
            1、 start_date 和 end_date 均不填， 返回距离当前日期的1000 根 K 线；
            2、 仅填 start_date， 当 start_date和最新日期之间的数据不足1000 根，返回 start_date 和最新日期之间的数据；如果数据超过 1000 根 K 线， 则返回距离当前日期的 1000 根 K线；
            3、 仅填 end_date ， 返 回end_date 之前存在的的最多1000 根 K 线；
            4、 start_date 和 end_date 均填充，返回该日期区间（闭区间）的数据，最多 1000 根。
            仅在 get_type=range 时有效。
        end_date:默认为当前日期；
            1、 start_date 和 end_date 均不填， 返回距离当前日期的1000 根 K 线；
            2、 仅填 start_date， 当 start_date和最新日期之间的数据不足1000 根，返回 start_date 和最新日期之间的数据；如果数据超过 1000 根 K 线， 则返回距离当前日期的 1000 根 K线；
            3、 仅填 end_date ， 返 回end_date 之前存在的的最多1000 根 K 线；
            4、 start_date 和 end_date 均填充，返回该日期区间（闭区间）的数据，最多 1000 根。
            仅在 get_type=range 时有效。

        '''
        #     https://sandbox.hscloud.cn/quote/v1/kline
        stock_code = self.get_stock_id_hson(stock_code)
        url = self.host_v1 + '/kline'
        params = dict(get_type=get_type,
                      prod_code=stock_code,
                      date=date,
                      data_count=data_count,
                      candle_period=candle_period,
                      candle_mode=candle_mode,
                      )
        if min_time is not None:
            params.setdefault("min_time", min_time)
        # if fields is not None:
        #     params['fields'] = fields
        data = self.http_get_hsopen(url, params)
        columns = data['data']['candle']['fields']
        row_data = data['data']['candle'][stock_code]
        df_data = DataFrame(row_data, columns=columns)
        df_data.set_index('min_time', inplace=True)
        return df_data

    def get_kline_day(self, stock_code
                      , end_date
                      , count
                      # , fields=['close', 'volume', 'open', 'high', 'low', 'pre_close']
                      # , skip_paused=True
                      # , fq='pre'
                      ):
        """
        获取日级别K线数据
        :param stock:
        :param end_date: 20170901
        :param count:
        :param frequency:
        :param fields:
        :param skip_paused:
        :param fq:
        :return:
            pd.DataFrame
            min_time  open_px  high_px  low_px  close_px  business_amount  business_balance
        0  20171019    28.12    28.66   27.89     28.26         16641711    16641711
        1  20171020    28.24    29.07   28.02     29.03         19339570    19339570
        """
        stock_code = self.get_stock_id_hson(stock_code)
        data = self.get_kline(stock_code, end_date, count)
        return data

    def get_kline_minute(self,
                         stock_code,
                         end_date,
                         end_time,
                         count,
                         # frequency='1m',
                         # fields=['close', 'volume', 'open', 'high', 'low','pre_close'],
                         # skip_paused=True,
                         fq='pre'
                         ):
        """
        获取日级别K线数据
        :param stock:
        :param end_date: 数据格式 20171102
        :param count:
        :param frequency:
        :param fields:
        :param skip_paused:
        :param fq:
        :return:
            pd.DataFrame
            min_time  open_px  high_px  low_px  close_px  business_amount  business_balance
        0  201711011451    28.09    28.11   28.09     28.11    52570        1477244
        1  201711011452    28.10    28.12   28.10     28.11    52600        1478546
        """
        stock_code = self.get_stock_id_hson(stock_code)

        df_data = self.get_kline(stock_code, end_date, count, candle_period=1, min_time=end_time)
        return df_data

    def get_stock_info(self, stock):
        '''
        获取证券信息
        证券代码、证券简称、上市日期、证券类型、证券市场、上市板块
        :param stock: 600549
        :return: {"issue_price":"11.6000","listed_date":"2002-11-07","listed_sector":"主板","secu_abbr":"厦门钨业","secu_category":"A股","secu_code":"600549","secu_market":"上海证券交易所"}
        issue_price 发行价
        listed_date 上市日期
        listed_sector 上市板块
        secu_code 证券代码
        secu_abbr 证券简称
        secu_category 证券类型
        secu_market 证券市场
        '''
        try:
            stock_info = self.get_stock_info_hson(stock)
            rs = {
                'issue_price': stock_info['issue_price'],
                'listed_date': stock_info['listed_date'],
                'listed_sector': stock_info['listed_sector'],
                'secu_code': stock_info['secu_code'],
                'secu_abbr': stock_info['secu_abbr'],
                'secu_category': stock_info['secu_category'],
                'secu_market': stock_info['secu_market']
            }
            return rs
        except:
            print("get_stock_info_of_stock exception ", traceback.print_exc())
            return {}

    def get_stock_info_hson(self, stock_code):
        stock_code = self.get_stock_id_hson(stock_code)
        url = self.host_info_v2 + '/query/f10_secu_info'
        params = {"en_prod_code": stock_code}
        data = self.http_get_hsopen(url, params)
        stock_info = data['data'][0]['20101013'][0][stock_code][0]
        return stock_info


# 实例化
hson_data = HsonData()

if __name__ == "__main__":
    print("想测试？去test文件夹下找对应测试类")



# coding:utf8
import logging
import logging.handlers

log = logging.getLogger('miceshare')
log.setLevel(logging.DEBUG)
log.propagate = False

fmt = logging.Formatter('%(asctime)s [%(levelname)s] %(filename)s %(lineno)s: %(message)s')
# ch = logging.StreamHandler()
# ch.setFormatter(fmt)
# log.handlers.append(ch)


LOG_FILE = 'logs/info.log'
handler = logging.handlers.RotatingFileHandler(LOG_FILE, maxBytes = 1024*1024*10, backupCount=10)

handler.setFormatter(fmt)
log.addHandler(handler)
log.setLevel(logging.DEBUG)



# coding: utf-8

import pandas as pd
from miceshare.db import mysql_access
import tushare as ts
from miceshare.util import vars
from miceshare.util.stock_utils import get_stock_code,get_stock_type


def import_all_securities():
    """
    导入股票列表数据
    :return:
    """
    all_securities = pd.read_csv("all_securities.csv")
    print(all_securities.head())
    for index, row in all_securities.iterrows():
        code = row[0]
        name = row[1]
        date = row[3]
        ex = get_stock_type(row[0])

        mysql_access.stock_dao.insert(code,name,date,ex)
    print("done")

def import_all_concept_classified():
    concept = ts.get_concept_classified()
    print(concept)
    for k,v in concept.iterrows():
        code = v['code']
        name = v['name']
        c_name = v['c_name']
        print(code,name,c_name)
        mysql_access.board_dao.insert(c_name,c_name,vars.Board_C)
        mysql_access.board_stock_dao.insert(code,c_name)

# import_all_securities
import_all_concept_classified()

# print(stock_dao.select('2017-11-22'))
# print(board_dao.select())
# print(board_stock_dao.select())


# coding=utf-8

__author__ = 'xf'
__versioninfo__ = (0, 0, 16)
__version__ = '.'.join(map(str, __versioninfo__))
__title__ = 'miceshare'

"""
for classifying data
"""
from miceshare.classifying import (get_stock_basics,get_industry_classified,
                                   get_concept_classified,get_concept_for_stock,get_concept_stocks,
                                       get_area_classified, get_gem_classified,
                                       get_sme_classified, get_st_classified,
                                       get_hs300s, get_sz50s, get_zz500s,
                                       get_terminated, get_suspended,
                                   insert_board,insert_board_stock)


import miceshare.db.sqlite_util
import miceshare.kaipan
import miceshare.db


# -*- coding:utf-8 -*-

"""
获取股票分类数据接口 

"""

import pandas as pd
from miceshare.db import mysql_access
import datetime
from functools import lru_cache
from miceshare.util.vars import *
from miceshare.util import vars


def get_industry_classified():
    """
        获取行业分类数据
    Returns
    -------
    DataFrame
        code :股票代码
        name :股票名称
        c_name :行业名称
    """
    pass

def get_board_list(channel = 0):
    """
    获取板块列表
    :return:
        DataFrame
            code:概念
            name:概念名称
            type:类型
            channel:来源
    """
    boards = mysql_access.board_dao.select()
    data = pd.DataFrame(boards, columns=['code', 'name','type','channel'])
    data = data[data['channel'] == channel]
    return data


@lru_cache(None)
def get_board_name(board_code):
    board_list = get_board_list()
    board_list[board_list['code'] == board_code]['name'].tolist()

def get_concept_stocks(concept,channel,date=None):
    """
    获取概念板块下的股票列表
    :param name: 板块名称
    :param channel:渠道1是同花顺
    :param date:股票上市日期，暂时无用
    :return:
    """
    return mysql_access.board_stock_dao.query_stocks_by_board(concept,vars.Board_C,channel)


@lru_cache(None)
def get_concept_for_stock(stock,channel=0):
    """
        获取概念列表
    Return
    --------
    ['概念名称']
    """
    concept = get_concept_classified()
    board_code_list = concept[concept['stock_code'] == stock]
    board_list = get_board_list(channel)
    m = pd.merge(board_code_list, board_list,left_on='board',right_on='code')
    return m['name'].tolist()

@lru_cache(None)
def get_concept_classified():
    """
        获取概念分类数据
    Return
    --------
    DataFrame
        code :股票代码
        name :股票名称
        c_name :概念名称
    """
    l = mysql_access.board_stock_dao.select()
    return pd.DataFrame(l, columns=['stock_code', 'board'])


def get_area_classified():
    """
        获取地域分类数据
    Return
    --------
    DataFrame
        code :股票代码
        name :股票名称
        area :地域名称
    """
    pass


def get_gem_classified():
    """
        获取创业板股票
    Return
    --------
    DataFrame
        code :股票代码
        name :股票名称
    """
    df = get_stock_basics()
    df = df.ix[df.code.str[0] == '3']
    df = df.sort_values('code').reset_index(drop=True)
    return df
    

def get_sme_classified():
    """
        获取中小板股票
    Return
    --------
    DataFrame
        code :股票代码
        name :股票名称
    """
    df = get_stock_basics()
    df = df.ix[df.code.str[0:3] == '002']
    df = df.sort_values('code').reset_index(drop=True)
    return df 

def get_st_classified():
    """
        获取风险警示板股票
    Return
    --------
    DataFrame
        code :股票代码
        name :股票名称
    """
    df = get_stock_basics
    df.reset_index(inplace=True)
    df = df.ix[df.name.str.contains('ST')]
    df = df.sort_values('code').reset_index(drop=True)
    return df 

@lru_cache(None)
def get_stock_basics(time=None):
    """
    查询股票列表
    :param time 截止上市日期
    :return:
    DataFrame
        code :股票代码
        name :股票名称
        start_date :日期
        exchange:市场 sz或sh
    """
    if time is None:
        time = datetime.datetime.now().date()
    l = mysql_access.stock_dao.select(time)
    return pd.DataFrame(l,columns=['code','name','start_date','exchange'])

def get_hs300s():
    """
    获取沪深300当前成份股及所占权重
    Return
    --------
    DataFrame
        code :股票代码
        name :股票名称
        date :日期
        weight:权重
    """
    pass


def get_sz50s():
    """
    获取上证50成份股
    Return
    --------
    DataFrame
        code :股票代码
        name :股票名称
    """
    pass


def get_zz500s():
    """
    获取中证500成份股
    Return
    --------
    DataFrame
        code :股票代码
        name :股票名称
    """
    pass


def get_terminated():
    """
    获取终止上市股票列表
    Return
    --------
    DataFrame
        code :股票代码
        name :股票名称
        oDate:上市日期
        tDate:终止上市日期 
    """
    pass


def get_suspended():
    """
    获取暂停上市股票列表
    Return
    --------
    DataFrame
        code :股票代码
        name :股票名称
        oDate:上市日期
        tDate:终止上市日期 
    """
    pass


def insert_board(code, name, type=Board_C, channel=0):
    """
    插入概念板块数据
    :param code:
    :param name:
    :return:
    """
    mysql_access.board_dao.insert(code, name, type,channel)

def insert_board_stock(stock_code,board_code):
    """
    插入股票和板块的关系
    :param stock_code:
    :param board_code:
    :return:
    """
    mysql_access.board_stock_dao.insert(stock_code, board_code)


# coding=utf-8
#

import tushare as ts
import pandas as pd
import sqlite3
import miceshare.configs as configs

data_path = configs.data_path


def init_data():
    df = ts.get_stock_basics()
    conn = sqlite3.connect(data_path + 'default')
    df.to_sql(name='basic', con=conn, if_exists='replace')


def query_all_stocks():
    conn = sqlite3.connect(data_path + 'default')
    return pd.read_sql('select * from basic', con=conn)


if __name__ == '__main__':
    # init_data()
    stocks = query_all_stocks()
    print(stocks.head())



# -*- coding:utf-8 -*-
"""
K线数据
"""

def get_k_stock_d():
    pass

def get_k_stock_m():
    pass


def get_k_index_d():
    pass

def get_k_index_m():
    pass

def get_k_board_d(code,start,end,count=30,autype='qfq',ktype='D'):
    """
    获取k线数据-板块
    ---------
    Parameters:
      code:string
                  板块代码 e.g. 885756
      start:string
                  和count选其一，开始日期 format：YYYY-MM-DD 为空时取上市首日
      end:string
                  结束日期 format：YYYY-MM-DD 为空时取最近一个交易日
      count:int
                  和star选其一，如果只指定结束日期，则使用个数
      autype:string
                  复权类型，qfq-前复权 hfq-后复权 None-不复权，默认为qfq
      ktype：string
                  数据类型，D=日k线 W=周 M=月 5=5分钟 15=15分钟 30=30分钟 60=60分钟，默认为D
    return
    -------
      DataFrame
          date 交易日期 (index)
          open 开盘价
          high  最高价
          close 收盘价
          low 最低价
          volume 成交量
          amount 成交额
          code 代码
    """




def get_k_board_d():
    pass


# coding: utf-8

import unittest
import miceshare


class MiceShareTest(unittest.TestCase):
    """
        测试miceshare接口
    """

    def test_get_get_stock_basics(self):
        df = miceshare.get_stock_basics()
        print(df.shape)
        print(df.head())

    def test_get_concept(self):
        c = miceshare.get_concept_classified()
        print(c[c['stock_code']=='600822']['board'].tolist())

        c = miceshare.get_concept_classified()
        print(c[c['board'] == '5G概念'])

        c = miceshare.get_concept_classified()
        print(c[c['board'] == '国企改革'])






# coding=utf-8
#

import json
import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import miceshare.db.sqlite_util as sql_util
import matplotlib.ticker as ticker


def build_df(r):
    s = json.loads(r)
    bsentrust = s['bsentrust']
    day = s['day']
    code = s['code']

    df = pd.DataFrame(bsentrust)
    cols = ['time', 'u1', 'u2', 'buy', 'sell']
    df.columns = cols
    df['std_sell'] = df['sell'].rolling(5).std()
    df['std_buy'] = df['buy'].rolling(5).std()
    # df['mean_sell'] = df['std_sell'].rolling(5).mean()
    # df['mean_buy'] = df['std_buy'].rolling(5).mean()
    return df


def plot_bs(code):
    data_type = 'bs'
    conn = sql_util.login(sql_util.get_db_name('bs'))
    table_name = 'bs_20171110'
    r = sql_util.query(conn, table_name, [code])
    df = build_df(r[1])
    N = len(df)
    ind = np.arange(N)
    print(N)

    def format_date(x, pos=None):
        thisind = np.clip(int(x + 0.5), 0, N - 1)
        return df['time'][thisind]

    fig, axes = plt.subplots(ncols=2, figsize=(8, 4))
    ax = axes[0]
    ax.plot(ind, df['sell'], label='sell')
    ax.plot(ind, df['buy'], label='buy')
    ax.xaxis.set_major_formatter(ticker.FuncFormatter(format_date))
    fig.autofmt_xdate()
    ax.legend(loc='best')
    ax.set_xlim(10, 230)
    plt.title(code)

    ax = axes[1]
    ax.plot(ind, df['std_sell'], label='std_sell')
    # ax.plot(ind, df['mean_sell'], label='mean_sell')
    ax.plot(ind, df['std_buy'], label='std_buy')
    # ax.plot(ind, df['mean_buy'], label='mean_buy')
    ax.xaxis.set_major_formatter(ticker.FuncFormatter(format_date))
    fig.autofmt_xdate()
    # ax.set_xlim(10, 230)
    # ax.set_ylim(-1000, 1500000)
    ax.legend(loc='best')
    plt.savefig('d:/tmp/' + table_name)
    plt.show()


if __name__ == '__main__':
    plot_bs('601108')



# coding=utf-8
#

import sqlite3
import datetime
from miceshare.util import vars

data_path = vars.sqlite3_data_path
create_sql = 'create table if not exists %s (code char(6) PRIMARY KEY, data text)'
select_sql = 'select * from %s'
select_which = 'select * from %s where code=?'
insert_sql = 'INSERT INTO %s VALUES (?,?)'


def get_db_name(data_type='default'):
    return data_type


def format_sql(sql, data):
    return sql % data


def get_table_name(data_type):
    now = datetime.date.today()
    if data_type == 'bs':
        return 'bs_' + now.strftime('%Y%m%d')


def login(db_name):
    conn = sqlite3.connect(data_path + db_name)
    return conn


def create_table(conn, sql):
    cursor = conn.cursor()
    cursor.execute(sql)
    print('change', cursor.rowcount)
    conn.commit()


def create_if_not_exists(conn, data_type):
    table_name = get_table_name(data_type)
    create_sql_ = format_sql(create_sql, table_name)
    print(create_sql_)
    create_table(conn, create_sql_)


def insert_data(conn, data_type, data):
    table_name = get_table_name(data_type)
    sql = format_sql(insert_sql, (table_name))
    insert(conn, sql, data)


def insert(conn, sql, data):
    try:
        cursor = conn.cursor()
        cursor.execute(sql, data)
        print('insert', cursor.rowcount)
        conn.commit()
    except:
        print('fail', data)


def query(conn, table_name, data=None):
    sql = None
    if data:
        sql = format_sql(select_which, (table_name))
    else:
        sql = format_sql(select_sql, (table_name))
    print(sql)
    cursor = conn.cursor()
    if data:
        print(sql, data)
        cursor.execute(sql, data)
    else:
        cursor.execute(sql)
    row = cursor.fetchone()
    return row


def query_all():
    conn = sqlite3.connect('d:/go/data/kpl')
    cursor = conn.cursor()
    cursor.execute('SELECT * FROM bs_20171109')
    for r in cursor.fetchall():
        print(r)


def dump_data(conn, data_type, data):
    sql_ = None
    if data_type == 'bs':
        _sql = format_sql(insert_sql, get_table_name(data_type))

    insert(conn, _sql, data)


if __name__ == '__main__':
    conn = login('bs')
    # create_if_not_exists(conn, 'bs')
    query1 = query(conn, 'bs_20171113')
    print(query1)



# coding: utf-8

import requests
import time
import json
import traceback
import datetime
import sys
import pandas as pd

'''
转换数字股票代码id,600136.XSHG->600136
'''


def get_stock_code(stock_code):
    code = stock_code
    idx = stock_code.find('.')
    if idx > 0:
        code = code[0:idx]
    return code


# 资金流计算
'''=============================================东财数据获取========================================================'''


# 参考地址 http://liam0205.me/2016/02/27/The-requests-library-in-Python/
def http_get_em(url, params):
    my_headers = {
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/48.0.2564.116 Safari/537.36',
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
        'Accept-Encoding': 'gzip',
        'Accept-Language': 'zh-CN,zh;q=0.8,en;q=0.6,zh-TW;q=0.4',
        'Content-Type': 'application/x-www-form-urlencoded'}
    try:
        r = requests.get(url, headers=my_headers, data=params, timeout=5)
        #         print(r.status_code, r.reason,r.text)
        return r.text
    except:
        return 'http get exception'


'''
get content between str1 and str2 in str
'''


def str_get_between(str, str1, str2):
    strOutput = str[str.find(str1) + len(str1):str.find(str2)]
    return strOutput


'''
转换成东财的股票代码id,600136.XSHG->6001361
'''


def get_stock_id_eastmoney(stock_code):
    code = stock_code
    idx = stock_code.find('.')
    if idx > 0:
        code = code[0:idx]
    if code.startswith("60"):
        code = code + "1";
    elif code.startswith("00"):
        code = code + "2";
    elif code.startswith("300"):
        code = code + "2";
    # print(code)
    return code


'''
聚宽的id
'''


def get_stock_id_jq(stock_code):
    code = stock_code
    idx = stock_code.find('.')
    if idx > 0:
        code = code[0:idx]
    if code.startswith("60"):
        code = code + ".XSHG";
    elif code.startswith("00"):
        code = code + ".XSHE";
    elif code.startswith("300"):
        code = code + ".XSHE";
    # print(code)
    return code


'''
http://ff.eastmoney.com/EM_CapitalFlowInterface/api/js?id=6003261&type=ff&check=MLBMS&cb=var%20aff_data=&js={(x)}&rtntype=3&_=1493254730313
每分钟的数据，按照下面的顺序的数组，比如：'-1232.0203,-456.3307,-775.6896,157.0809,1074.939'
今日主力净流入：-1232.0200万元,主力净比：-1.25%
今日超大单净流入：-456.3300万元,超大单净比：-0.46%
今日大单净流入：-775.6900万元,大单净比：-0.79%
今日中单净流入：157.0800万元,中单净比：0.16%
今日小单净流入：1074.9400万元,小单净比：1.09%
'''


def parse_money_flow_minute_now(stock):
    now = str(time.time())
    stock = get_stock_id_eastmoney(stock)
    url = 'http://ff.eastmoney.com/EM_CapitalFlowInterface/api/js?id=' + stock + '&type=ff&check=MLBMS&cb=var%20aff_data=&js={(x)}&rtntype=3&acces_token=1942f5da9b46b069953c873404aad4b5&_=' + now
    #    print(url)
    html = http_get_em(url, '')
    data = str_get_between(html, '"ya":', '}')
    #     print(data)
    data = json.loads(data)
    #     print(data)
    return data


'''
获取资金流-历史-天拆分
'''


def get_money_flow_day(stock, date):
    mf = jqdata.get_money_flow(stock, end_date=date, count=1)
    #     print(mf)
    #     print(mf.net_amount_xl+mf.net_amount_l)
    return mf.net_amount_xl + mf.net_amount_l


'''=============================================kaipan数据获取========================================================'''
'''
开盘的数据获取，整理接口：
获取资金流-历史
获取资金流-当天实时
获取分钟K线-当天实时
获取盘口-当天实时

注意：使用自己账户信息获取数据
'''


class KaipanData():
    def __init__(self):
        pass

    g_token = '86402db4dedc31239b54eb6763b57d47'  # 有些需要权限的需要抓手机得到
    g_url_now = 'https://hq.kaipanla.com/w1/api/index.php'  # 实时接口
    g_url_his = 'https://his.kaipanla.com/w1/api/index.php'  # 历史接口
    # 统一模拟请求头
    my_headers = {'Content-type': 'multipart/form-data; boundary=xds~Boundary',
                  'Accept': '/',
                  'Content-Length': '536',
                  'User-Agent': '%E5%BC%80%E7%9B%98%E5%95%A6/6 CFNetwork/808.2.16 Darwin/16.3.0',
                  'Accept-Language': 'zh-cn',
                  'Accept-Encoding': 'gzip, deflate'
                  }
    my_headers_his = {'Host': 'his.kaipanla.com'}
    my_headers_his.update(my_headers)
    my_headers_now = {'Host': 'hq.kaipanla.com'}
    my_headers_now.update(my_headers)

    '''
    转换数字股票代码id,600136.XSHG->600136
    '''

    def get_stock_code(self, stock_code):
        code = stock_code
        idx = stock_code.find('.')
        if idx > 0:
            code = code[0:idx]
        return code

    '''
    统一请求
    '''

    def post(self, url, headers, body, attr=None, timeout=5):
        # body的另外一种方式，待尝试，简化代码
        #     files={"UserID":(None,'103935'),"a":(None,"GetStockChouMa"),"Token":(None,"f5f65817cf08a406918ea918690d18ea"),
        # "c":(None,"StockL2History"),"StockID":(None,stock),"Day":(None,date),"apiv":(None,"w4")}
        try:
            r = requests.post(url, headers=headers, data=body, timeout=timeout)
            if (r.status_code == 200):
                rs_josn = r.json()
                if None != attr:
                    return rs_josn[attr]
                else:
                    return rs_josn
        except Exception as e:
            print("longge http post exception ", e)
            return []

    '''获取资金流数据-历史
    [-49708600, 18441100, 3754260, 27513200, 14, 19104200, 264, 1183620000, '15:00']
    大于50万，30-50万，10-30万，小于10万，对倒次数，对待金额，异动次数，成交额，时间
    '''

    def get_money_flow_m5_hist_kaipan(self, stock, date):
        body = """\
--xds~Boundary
Content-Disposition: form-data; name="a"

GetStockChouMa
--xds~Boundary
Content-Disposition: form-data; name="c"

StockL2History
--xds~Boundary
Content-Disposition: form-data; name="StockID"

%s
--xds~Boundary
Content-Disposition: form-data; name="UserID"

103935
--xds~Boundary
Content-Disposition: form-data; name="Token"

%s
--xds~Boundary
Content-Disposition: form-data; name="Day"

%s
--xds~Boundary
Content-Disposition: form-data; name="apiv"

w4

""" % (stock, self.g_token, date)
        return self.post(self.g_url_his, self.my_headers_his, body, attr='List')

    '''
    获取资金流数据-当天-5分钟拆分，每分钟更新一次数据，不够的计入当前
    stock : "002350"
    [-49708600, 18441100, 3754260, 27513200, 14, 19104200, 264, 1183620000, '15:00']
    大于50万，30-50万，10-30万，小于10万，对倒次数，对待金额，异动次数，成交额，时间
    '''

    def get_money_flow_m5_now_kaipan(self, stock):
        stock = self.get_stock_code(stock)
        body = """\
--xds~Boundary
Content-Disposition: form-data; name="UserID"

103935
--xds~Boundary
Content-Disposition: form-data; name="a"

StockChouMaByTimeNew
--xds~Boundary
Content-Disposition: form-data; name="Token"

%s
--xds~Boundary
Content-Disposition: form-data; name="c"

StockYiDongKanPan
--xds~Boundary
Content-Disposition: form-data; name="StockID"

%s
""" % (self.g_token, stock)
        return self.post(self.g_url_now, self.my_headers_now, body, attr='List')

    '''
    获取今天的最新5分钟主力资金流数据=特大单+大单
    '''

    def get_money_flow_m5_kaipan(self, stock, default_value=0):
        try:
            mm_m5 = self.get_money_flow_m5_now_kaipan(stock)
            for i in range(-1, -len(mm_m5), -1):
                c_mm = mm_m5[i]
                print(c_mm)
                large_mf = c_mm[0] + c_mm[1]
                if large_mf != 0:
                    return round(large_mf / 10000, 2)
            return 0
        except:
            print("get_money_flow_m5_kaipan exception ", traceback.print_exc())
            return default_value

    '''
    分钟K线图，返回数据如下：
    {
        "trend": [
            ["09:30", 32.35, 32.3716, 3032, 2],
            ["09:31", 32.27, 32.3475, 1804, 0],
            ["09:32", 32.23, 32.3119, 2594, 0],
            ...
            ["15:00", 32.47, 32.4896, 12, 2]
        ],
        "hprice": 32.74,
        "lprice": 32.22,
        "preclose_px": 32.45,
        "day": "20170922",
        "time": "1506063702",
        "code": "600340",
        "errcode": 0
    }
    '''

    def get_stock_trend_m1_now_kaipan(self, stock):
        stock = self.get_stock_code(stock)
        body = """\
--xds~Boundary
Content-Disposition: form-data; name="a"

GetStockTrend
--xds~Boundary
Content-Disposition: form-data; name="c"

StockL2Data
--xds~Boundary
Content-Disposition: form-data; name="StockID"

%s
--xds~Boundary
Content-Disposition: form-data; name="UserID"

103935
--xds~Boundary
Content-Disposition: form-data; name="DeviceID"

0f1ac73919caf7c55ecf23ffe6624d2d344faeae
--xds~Boundary
Content-Disposition: form-data; name="Token"

%s
--xds~Boundary
Content-Disposition: form-data; name="apiv"

w4
""" % (stock, self.g_token)
        return self.post(self.g_url_now, self.my_headers_now, body)

    '''
    盘口数据，返回结果如下
    {
        "day": 20170922,
        "code": "600340",
        "name": "\u534e\u590f\u5e78\u798f",
        "preclose_px": 32.45,#昨天收盘价
        "status": 86,
        "real": {
            "time": 150001000,
            "last_px": 32.47,#当前价
            "px_change": 0.02,#当前价格变动
            "px_change_rate": 0.06,#当前价格变动幅度
            "high_px": 32.74,#最高价
            "low_px": 32.22,#最低价
            "open_px": 32.36,#开盘价
            "avg_px": 32.4896,#均价
            "turnover_ratio": "0.66",#换手率
            "total_amount": 195363,#成交量总手
            "total_turnover": 634727980,#成交量总金额
            "vol_ratio": 0.49,#量比
            "up_px": 35.7,#涨停价
            "down_px": 29.21,#跌停价
            "amplitude": 1.6,#振幅
            "entrust_rate": -41.1,#委比
            "amount_in": 117837,#内盘
            "amount_out": 77525,#外盘
            "pe_rate": 8.94,#市盈率
            "dyn_pb_rate": 3.62,#市净率
            "circulation_amount": 2954946709,#流通股本
            "circulation_value": 95947112448,#流通市值
            "total_shares": 2954946709,#流通股本
            "market_value": 95947112448,#流通市值
            "actualcirculation_value": 40671980966#实际市值
        },
        "weituo": {
            "totals": 45940,
            "avg_askpx": 33.697,
            "s10": [0, 0],
            "s9": [0, 0],
            "s8": [0, 0],
            "s7": [0, 0],
            "s6": [0, 0],
            "s5": [32.52, 58],
            "s4": [32.51, 104],
            "s3": [32.5, 234],
            "s2": [32.49, 165],
            "s1": [32.48, 175],
            "totalb": 19175,
            "avg_bidpx": 31.96,
            "b1": [32.47, 25],
            "b2": [32.46, 31],
            "b3": [32.45, 405],
            "b4": [32.44, 78],
            "b5": [32.43, 159],
            "b6": [0, 0],
            "b7": [0, 0],
            "b8": [0, 0],
            "b9": [0, 0],
            "b10": [0, 0]
        },
        "errcode": 0,
        "ICon": 1,
        "Gang": ""
    }
    '''

    def get_stock_pankou_now_kaipan(self, stock):
        stock = self.get_stock_code(stock)
        body = """\
--xds~Boundary
Content-Disposition: form-data; name="a"

GetStockPanKou
--xds~Boundary
Content-Disposition: form-data; name="c"

StockL2Data
--xds~Boundary
Content-Disposition: form-data; name="StockID"

%s
--xds~Boundary
Content-Disposition: form-data; name="UserID"

103935
--xds~Boundary
Content-Disposition: form-data; name="DeviceID"

0f1ac73919caf7c55ecf23ffe6624d2d344faeae
--xds~Boundary
Content-Disposition: form-data; name="Token"

%s
--xds~Boundary
Content-Disposition: form-data; name="apiv"

w4

""" % (stock, self.g_token)
        return self.post(self.g_url_now, self.my_headers_now, body)

    '''
    获取日K数据
    "x": ["20151210", "20151211", ...] 日期
    "y": [
            [7.07, 8.37, 8.37, 7.07],#价格数据
            ...
            开盘，收盘，最高，最低
            [43.3, 42.58, 43.98, 42.27],
        ]
    "vol": [93, 43...
    "turnover": [0.04, 0.02,...
    '''

    def get_stock_kline_his_kaipan(self, stock):
        url = 'https://his.kaipanla.com/w1/api/index.php?apiv=w4'
        headers = {'Host': 'his.kaipanla.com',
                   'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
                   'Origin': 'https://page.kaipanla.com',
                   'Accept-Encoding': 'gzip, deflate',
                   'Connection': 'keep-alive',
                   'Accept': 'application/json, text/javascript, */*; q=0.01',
                   'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 10_3_3 like Mac OS X) AppleWebKit/603.3.8 (KHTML, like Gecko) Mobile/14G60',
                   'Referer': 'https://page.kaipanla.com/w4/web/StockDetails4_ios.html?SID=%s' % (stock),
                   'Accept-Language': 'zh-cn'
                   }
        stock = self.get_stock_code(stock)
        body = "StockID=%s&Type=d&Index=0&st=400&c=StockLineData&a=GetKLineDay&UserID=103935&Token=%s" % (stock, self.g_token)
        return self.post(url, headers, body)

    '''
    获取市场风口-板块数据
    date 格式：20171009
    is_hist：历史数据则设置为True否则获取到最新数据，不管date参数
    返回数据：
        [
            ["\u533b\u7597\u6539\u9769", "2100"], #板块名称，计算指数
            ["\u6b21\u65b0\u80a1", "488.42"],
            ......
        ]
    '''

    def get_market_fire_board_kaipan(self, date, is_his=False):
        if is_his:
            url = 'https://his.kaipanla.com/w1/api/index.php?apiv=w4'
            headers = {'Host': 'his.kaipanla.com',
                       'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
                       'Origin': 'https://page.kaipanla.com',
                       'Accept-Encoding': 'gzip, deflate',
                       'Connection': 'keep-alive',
                       'Accept': 'application/json, text/javascript, */*; q=0.01',
                       'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 10_3_3 like Mac OS X) AppleWebKit/603.3.8 (KHTML, like Gecko) Mobile/14G60',
                       'Referer': 'https://page.kaipanla.com/w4/web/capitalSpYD_ios.html',
                       'Accept-Language': 'zh-cn'
                       }
            body = "c=StockFengKData&a=GetFengKYDPlate&Day=%s&UserID=103935&Token=%s" % (date, self.g_token)
            return self.post(url, headers, body, attr='List')
        else:
            url = 'https://hq.kaipanla.com/w1/api/index.php?apiv=w4'
            headers = {'Host': 'hq.kaipanla.com',
                       'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
                       'Origin': 'https://page.kaipanla.com',
                       'Accept-Encoding': 'gzip, deflate',
                       'Connection': 'keep-alive',
                       'Accept': 'application/json, text/javascript, */*; q=0.01',
                       'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 10_3_3 like Mac OS X) AppleWebKit/603.3.8 (KHTML, like Gecko) Mobile/14G60',
                       'Referer': 'https://page.kaipanla.com/w4/web/capitalSpYD_ios.html',
                       'Accept-Language': 'zh-cn'
                       }
            body = "c=StockFengKData&a=GetFengKYDPlate&UserID=103935&Token=%s" % (self.g_token)
            return self.post(url, headers, body, attr='List')

    '''
    根据风口板块获取风口股票数据
    date 格式：20171009
    is_hist：历史数据则设置为True否则获取到最新数据，不管date参数
    返回数据格式：
        [{
            "StockID": "002223", #代码
            "StockName": "\u9c7c\u8dc3\u533b\u7597", #名称
            "jiage": "22.9600", #价格
            "zhangfu": "10.01%", #涨幅
            "sjltp": 13525339080, #实际流通市值
            "ZJBuy": 161131150, #主力买入
            "ZJSell": 105667318, #主力卖出
            "ZJJE": 55463832,#主力净额
            "GN": "\u533b\u7597\u6539\u9769", #板块
            "GN2": "\u533b\u7597\u6539\u9769", #板块
            "gang": "",
            "ZTState": 1,
            "ZTTime": "1507618674"}
            ...
            ]
    '''

    def get_market_fire_stock_of_board_kaipan(self, board, date, is_his=False):
        if is_his:
            url = 'https://his.kaipanla.com/w1/api/index.php?apiv=w4'
            headers = {'Host': 'his.kaipanla.com',
                       'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
                       'Origin': 'https://page.kaipanla.com',
                       'Accept-Encoding': 'gzip, deflate',
                       'Connection': 'keep-alive',
                       'Accept': 'application/json, text/javascript, */*; q=0.01',
                       'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 10_3_3 like Mac OS X) AppleWebKit/603.3.8 (KHTML, like Gecko) Mobile/14G60',
                       'Referer': 'https://page.kaipanla.com/w4/web/capitalSpGN_ios.html?n=%s&date=%s' % (board, date),
                       'Accept-Language': 'zh-cn'
                       }
            body = "c=StockFengKData&a=GetFengKYDPlateInfo&Plate=%s&filter=0&Jump=1&Day=%s&UserID=103935&Token=%s" % (board, date, self.g_token)
            return self.post(url, headers, body, attr='List')
        else:
            url = 'https://hq.kaipanla.com/w1/api/index.php?apiv=w4'
            headers = {'Host': 'hq.kaipanla.com',
                       'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
                       'Origin': 'https://page.kaipanla.com',
                       'Accept-Encoding': 'gzip, deflate',
                       'Connection': 'keep-alive',
                       'Accept': 'application/json, text/javascript, */*; q=0.01',
                       'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 10_3_3 like Mac OS X) AppleWebKit/603.3.8 (KHTML, like Gecko) Mobile/14G60',
                       'Referer': 'https://page.kaipanla.com/w4/web/capitalSpGN_ios.html?n=%s' % (board),
                       'Accept-Language': 'zh-cn'
                       }
            body = "c=StockFengKData&a=GetFengKYDPlateInfo&Plate=%s&filter=0&Jump=1&UserID=103935&Token=%s" % (board, self.g_token)
            return self.post(url, headers, body, attr='List')

    '''
    委卖委买数据
    {
        "bsentrust": [
            [
                "09:30",
                3500,
                12526,
                10440500,
                39746696
            ],
            [
                "09:31",
                6057,
                15429,
                18075904,
                48897448
            ],
            ...
        ],
        "maxentrust": 39146,
        "minentrust": 3500,
        "minentrustdiff": -21267,
        "maxentrustdiff": -1614,
        "maxturnover": 123139824,
        "mineturnover": 10440500,
        "mineturnoverdiff": -69024772,
        "maxturnoverdiff": -9686296,
        "day": "20171110",
        "time": "1510470100",
        "code": "600340",
        "errcode": 0,
        "time2": 1510476870
    }
    '''
    def get_bs(self, code):
        stock = self.get_stock_code(code)
        data = {'c': 'StockL2Data', 'a': 'GetStockBsvolume', 'apiv': 'w5'}
        data['StockID'] = stock
        url = "http://hq.kaipanla.com/w1/api/index.php"
        headers = {"User-Agent": "Dalvik/2.1.0 (Linux; U; Android 7.0; BLN-AL10 Build/HONORBLN-AL10)",
                   "Host": "hq.kaipanla.com"
                   }
        post = requests.post(url, data=data, headers=headers)
        if post.status_code == 200:
            return post.text
        print('get bs fail', post.status_code)
        return None


    def get_bs_df(self, code):
        json = get_bs(code)
        if json:
            ret = json['bsentrust']
            df = pd.DataFrame(ret)
            return df
        print('not found')
        return None


# 为了兼容直接暴露以下方法
kaipan_data = KaipanData()

# 委卖委买数据
def get_bs(stock):
    return kaipan_data.get_bs(stock)

# 今天分钟K线明细
def get_stock_trend_m1_now_kaipan(stock):
    return kaipan_data.get_stock_trend_m1_now_kaipan(stock)


# 今天所有资金流数据明细
def get_money_flow_m5_now_kaipan(stock):
    return kaipan_data.get_money_flow_m5_now_kaipan(stock)


# 今天最新数据主力资金
def get_money_flow_m5_kaipan(stock):
    return kaipan_data.get_money_flow_m5_kaipan(stock)


# 历史某天的资金流数据明细
def get_money_flow_m5_hist_kaipan(stock, date):
    return kaipan_data.get_money_flow_m5_hist_kaipan(stock, date)


# 今天的盘口数据
def get_stock_pankou_now_kaipan(stock):
    return kaipan_data.get_stock_pankou_now_kaipan(stock)


def get_stock_kline_his_kaipan(stock):
    return kaipan_data.get_stock_kline_his_kaipan(stock)


# 板块风口排名数据
def get_market_fire_board_kaipan(date, is_his=False):
    return kaipan_data.get_market_fire_board_kaipan(date, is_his)


# 根据风口板块获取对应的股票
def get_market_fire_stock_of_board_kaipan(board, date, is_his=False):
    return kaipan_data.get_market_fire_stock_of_board_kaipan(board, date, is_his)


if __name__ == "__main__":
    '''测试'''
    stock = "600340"
    # mf = get_stock_trend_m1_now_kaipan(stock)
    # print(mf)

    # mf = get_money_flow_m5_kaipan(stock)
    # print(mf)

    # mf = get_money_flow_m5_now_kaipan(stock)
    # print(mf)

    # mf = get_money_flow_m5_hist_kaipan(stock,"2017005")
    # print(mf)

    # mf = get_stock_pankou_now_kaipan(stock)
    # print(mf)

    # mf = get_stock_kline_his_kaipan(stock)
    # print(mf)

    # mf = get_market_fire_board_kaipan('20171010')
    # print(mf)

    # mf = get_market_fire_stock_of_board_kaipan('5G', '20171010', is_his=False)
    mf = get_bs(stock)
    print(mf)




# coding: utf-8

from tornado import gen
import datetime
from miceshare.util import vars
import pandas as pd

# log = logger.logger
POOL = None

'''
dao操作基类，用于统一操作
'''
class dao_base(object):

    is_dev = True

    if is_dev:
        dbconfig = dict(
            database='smart_stock',
            user='stock',
            password='1234567',
            host='127.0.0.1',
            port=3306,
            charset="utf8"
        )
    else:
        dbconfig = dict(
            database='antman_stock',
            user='antman',
            password='7SZqMjEbrQ7WvNU8',
            host='rm-wz9768lm302k078o7.mysql.rds.aliyuncs.com',
            port=3306,
            charset="utf8"
        )

    def _get_conn(self):
        global POOL
        if POOL is None:
            print("init database pool")
            import mysql.connector.pooling
            POOL = mysql.connector.pooling.MySQLConnectionPool(pool_name="mypool", pool_size=10, **self.dbconfig)
            print("init db successed")
        con = POOL.get_connection()
        return con

    def _close_conn(self, conn):
        conn.close()

    def excute(self,query,params):
        con = self._get_conn()
        try:
            c = con.cursor()
            c.execute(query, params)
            con.commit()
            rows_affected = c.rowcount
            c.close()
            return rows_affected
        finally:
            con.close()

    def query(self, query, params):
        print(query,params)
        conn = self._get_conn()
        list = []
        try:
            cur = conn.cursor()
            cur.execute(query, params)
            for row in cur:
                # log.info([row[0], row[1]])
                list.append(row)
            cur.close()
        finally:
            conn.close()

        return list

    #插入一行
    def insert_row(self,query,params):
        return self.excute(query,params)

    #判断是否存在
    def exist(self,query,params):
        count = self.count(query,params)
        return count>0

    def count(self,query, params):
        con = self._get_conn()
        count = 0
        try:
            c = con.cursor()
            c.execute(query, params)
            # rows_affected = c.rowcount
            count = c.fetchone()[0]
        finally:
            con.close()
        return count

    def delete(self, query, params):
        self.excute(query, params)


class stock_dao(dao_base):

    def __init__(self):
        # print("init stock")
        super(stock_dao,self).__init__()
        self.name='stock'

    def insert(self,code,name,date,ex):
        query = "INSERT INTO stock(code,name,time,exchange) VALUE (%s,%s,%s,%s)"
        params = [code,name,date,ex]
        if self.is_exist(code):
            return 0
        return self.insert_row(query,params)

    def is_exist(self,code):
        query = "select count(code) from stock where code=%s"
        params = [code]
        is_exist = self.exist(query, params)
        return is_exist

    def select(self,time):
        query = "select code,name,time,exchange from stock where time<=%s"
        params = [time]
        return self.query(query,params)


class board_dao(dao_base):

    def __init__(self):
        # print("init stock")
        super(board_dao,self).__init__()
        self.name='board'

    def insert(self,code,name,type,channel):
        query = "INSERT INTO board(code,name,type,channel) VALUE (%s,%s,%s,%s)"
        params = [code,name,type,channel]
        if self.is_exist(code):
            return 0
        return self.insert_row(query,params)

    def is_exist(self,code):
        query = "select count(code) from board where code=%s"
        params = [code]
        is_exist = self.exist(query, params)
        return is_exist

    def select(self):
        query = "select code,name,type,channel from board"
        params = None
        return self.query(query,params)

class board_stock_dao(dao_base):

    def __init__(self):
        # print("init stock")
        super(board_stock_dao,self).__init__()
        self.name='board_stock'

    def insert(self,stock_code,board_code):
        query = "INSERT INTO board_stock(stock_code,board_code) VALUE (%s,%s)"
        params = [stock_code,board_code]
        if self.is_exist(stock_code,board_code):
            return 0
        return self.insert_row(query,params)

    def is_exist(self,stock_code,board_code):
        query = "select count(stock_code) from board_stock where stock_code=%s and board_code=%s"
        params = [stock_code,board_code]
        is_exist = self.exist(query, params)
        return is_exist

    def select(self):
        query = "select stock_code,board_code from board_stock"
        params = None
        return self.query(query,params)

    def query_stocks_by_board(self,board_name,board_type=vars.Board_C,channel=1):
        query = "select A.stock_code from board_stock A LEFT JOIN board B on A.board_code=B.code WHERE B.name=%s and B.type=%s and B.channel=%s"
        params = [board_name,board_type,channel]
        l = self.query(query, params)
        return pd.DataFrame(l, columns=['stock_code'])['stock_code'].values



stock_dao = stock_dao()
board_dao = board_dao()
board_stock_dao = board_stock_dao()

if __name__ == '__main__':
    s = board_stock_dao.query_stocks_by_board('基金重仓',vars.Board_C,0)
    print(s)
# print(stock_dao.is_exist(123))
# stock_dao.insert(12346,'abc','2017-09-09','sh')
# stock_dao.insert(12345,'abc','2017-09-09','sh')
# print(stock_dao.select('2017-09-09'))


# coding: utf-8

import tushare

class SixQuantData(object):
    def __init__(self):
        pass

    def get_stock_list(self,date):
        pass



sq_data = SixQuantData()


tushare.get_concept_classified()
tushare.get_k_data()


# coding=utf-8
#

import requests
import sqlite3
import datetime
import pandas as pd
import logging


logging.basicConfig(level=logging.INFO,
                    filename='/tmp/stock.log',
                    filemode='w',
                    format='%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s: %(message)s')

def get_bs(code):
    data = {'c': 'StockL2Data', 'a': 'GetStockBsvolume', 'apiv': 'w5'}
    data['StockID'] = code
    url = "http://hq.kaipanla.com/w1/api/index.php"
    headers = {"User-Agent": "Dalvik/2.1.0 (Linux; U; Android 7.0; BLN-AL10 Build/HONORBLN-AL10)",
               "Host": "hq.kaipanla.com"
               }
    post = requests.post(url, data=data, headers=headers)
    try:
        if post.status_code == 200:
            return post.text
    except:
        logging.info('get bs fail %s', code)
    return None


def create_table(conn, sql):
    cursor = conn.cursor()
    cursor.execute(sql)
    logging.info('init table %s', cursor.rowcount)
    conn.commit()


def insert(conn, sql, data):
    try:
        if data:
            cursor = conn.cursor()
            cursor.execute(sql, data)
            logging.info('insert %s', cursor.rowcount)
            conn.commit()
    except:
        logging.info('fail %s', data)


if __name__ == '__main__':
    now = datetime.datetime.now()
    logging.info('start dump %s', now)
    conn = sqlite3.connect('/stock/data/bs_' + now.strftime('%Y%m'))
    table_name = 'bs_' + now.strftime('%Y%m%d')
    create_sql = 'create table if not exists %s (code char(6) PRIMARY KEY, data text)'
    insert_sql = 'INSERT INTO %s VALUES (?,?)'
    create_sql_ = create_sql % (table_name)
    create_table(conn, create_sql_)
    logging.info('create table: %s', table_name)
    conn_def = sqlite3.connect('/stock/data/default')
    stocks = pd.read_sql('select * from basic', con=conn_def)
    insert_sql_ = insert_sql % (table_name)
    for code in stocks['code']:
        logging.info('dump %s', code)
        bs = get_bs(code)
        insert(conn, insert_sql_, (code, bs))



# coding: utf-8


def get_stock_code(stock_code):
    """
    转换数字股票代码id,600136.XSHG->600136
    :param stock_code:
    :return:
    """
    code = stock_code
    idx = stock_code.find('.')
    if idx > 0:
        code = code[0:idx]
    return code

def get_stock_type(stock_code):
    """
    TODO 抽到工具类
    判断股票ID对应的证券市场
    匹配规则
    ['50', '51', '60', '90', '110'] 为 sh
    ['00', '13', '18', '15', '16', '18', '20', '30', '39', '115'] 为 sz
    ['5', '6', '9'] 开头的为 sh， 其余为 sz
    :param stock_code:股票ID, 若以 'sz', 'sh' 开头直接返回对应类型，否则使用内置规则判断
    :return 'sh' or 'sz'"""
    stock_code = str(stock_code)
    if stock_code.startswith(('sh', 'sz')):
        return stock_code[:2]
    if stock_code.startswith(('50', '51', '60', '73', '90', '110', '113', '132', '204', '78')):
        return 'sh'
    if stock_code.startswith(('00', '13', '18', '15', '16', '18', '20', '30', '39', '115', '1318')):
        return 'sz'
    if stock_code.startswith(('5', '6', '9')):
        return 'sh'
    return 'sz'